@@ -1,5 +1,5 @@
 /* Swfdec
- * Copyright (C) 2007 Benjamin Otte <otte@gnome.org>
+ * Copyright (C) 2007-2008 Benjamin Otte <otte@gnome.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -123,14 +123,82 @@ typedef struct {
   guint			refcount;	/* refcount - misused for recursion detection */
 } SwfdecAsWatch;
 
-G_DEFINE_TYPE (SwfdecAsObject, swfdec_as_object, G_TYPE_OBJECT)
+G_DEFINE_TYPE (SwfdecAsObject, swfdec_as_object, SWFDEC_TYPE_GC_OBJECT)
+
+static gboolean
+swfdec_as_watch_can_recurse (SwfdecAsWatch *watch)
+{
+  guint version;
+
+  version = swfdec_gc_object_get_context (watch->watch)->version;
+  if (version <= 6) {
+    return watch->refcount <= 1;
+  } else {
+    return watch->refcount <= 64 + 1;
+  }
+}
+
+static void
+swfdec_as_watch_ref (SwfdecAsWatch *watch)
+{
+  watch->refcount++;
+}
+
+static void
+swfdec_as_watch_unref (SwfdecAsWatch *watch)
+{
+  watch->refcount--;
+  if (watch->refcount == 0) {
+    swfdec_as_context_unuse_mem (swfdec_gc_object_get_context (watch->watch), 
+	sizeof (SwfdecAsWatch));
+    g_slice_free (SwfdecAsWatch, watch);
+  }
+}
+
+/* This is a huge hack design-wise, but we can't use watch->watch, 
+ * it might be gone already */
+static gboolean
+swfdec_as_object_steal_watches (gpointer key, gpointer value, gpointer object)
+{
+  SwfdecAsWatch *watch = value;
+
+  g_assert (watch->refcount == 1);
+  watch->watch = (SwfdecAsFunction *) object;
+  swfdec_as_watch_unref (watch);
+  return TRUE;
+}
+
+static void
+swfdec_as_object_free_property (gpointer key, gpointer value, gpointer data)
+{
+  SwfdecAsObject *object = data;
+
+  swfdec_as_context_unuse_mem (swfdec_gc_object_get_context (object), sizeof (SwfdecAsVariable));
+  g_slice_free (SwfdecAsVariable, value);
+}
 
 static void
 swfdec_as_object_dispose (GObject *gobject)
 {
+  SwfdecAsContext *context = swfdec_gc_object_get_context (gobject);
   SwfdecAsObject *object = SWFDEC_AS_OBJECT (gobject);
 
-  g_assert (object->properties == NULL);
+  if (context->debugger) {
+    SwfdecAsDebuggerClass *klass = SWFDEC_AS_DEBUGGER_GET_CLASS (context->debugger);
+    if (klass->remove)
+      klass->remove (context->debugger, context, object);
+  }
+
+  if (object->properties) {
+    g_hash_table_foreach (object->properties, swfdec_as_object_free_property, object);
+    g_hash_table_destroy (object->properties);
+    object->properties = NULL;
+  }
+  if (object->watches) {
+    g_hash_table_foreach_steal (object->watches, swfdec_as_object_steal_watches, object);
+    g_hash_table_destroy (object->watches);
+    object->watches = NULL;
+  }
   g_slist_free (object->interfaces);
   object->interfaces = NULL;
 
@@ -138,44 +206,41 @@ swfdec_as_object_dispose (GObject *gobject)
 }
 
 static void
-swfdec_as_object_mark_property (gpointer key, gpointer value, gpointer unused)
+swfdec_gc_object_mark_property (gpointer key, gpointer value, gpointer unused)
 {
   SwfdecAsVariable *var = value;
 
   swfdec_as_string_mark (key);
   if (var->get) {
-    swfdec_as_object_mark (SWFDEC_AS_OBJECT (var->get));
+    swfdec_gc_object_mark (var->get);
     if (var->set)
-      swfdec_as_object_mark (SWFDEC_AS_OBJECT (var->set));
+      swfdec_gc_object_mark (var->set);
   } else {
     swfdec_as_value_mark (&var->value);
   }
 }
 
 static void
-swfdec_as_object_mark_watch (gpointer key, gpointer value, gpointer unused)
+swfdec_gc_object_mark_watch (gpointer key, gpointer value, gpointer unused)
 {
   SwfdecAsWatch *watch = value;
 
   swfdec_as_string_mark (key);
-  swfdec_as_object_mark (SWFDEC_AS_OBJECT (watch->watch));
+  swfdec_gc_object_mark (watch->watch);
   swfdec_as_value_mark (&watch->watch_data);
 }
 
 static void
-swfdec_as_object_do_mark (SwfdecAsObject *object)
+swfdec_as_object_mark (SwfdecGcObject *gc)
 {
-  if (object->prototype)
-    swfdec_as_object_mark (object->prototype);
-  g_hash_table_foreach (object->properties, swfdec_as_object_mark_property, NULL);
-  if (object->watches)
-    g_hash_table_foreach (object->watches, swfdec_as_object_mark_watch, NULL);
-  g_slist_foreach (object->interfaces, (GFunc) swfdec_as_object_mark, NULL); 
-}
+  SwfdecAsObject *object = SWFDEC_AS_OBJECT (gc);
 
-static void
-swfdec_as_object_do_add (SwfdecAsObject *object)
-{
+  if (object->prototype)
+    swfdec_gc_object_mark (object->prototype);
+  g_hash_table_foreach (object->properties, swfdec_gc_object_mark_property, NULL);
+  if (object->watches)
+    g_hash_table_foreach (object->watches, swfdec_gc_object_mark_watch, NULL);
+  g_slist_foreach (object->interfaces, (GFunc) swfdec_gc_object_mark, NULL); 
 }
 
 static gboolean
@@ -195,7 +260,7 @@ swfdec_as_object_hash_lookup (SwfdecAsObject *object, const char *variable)
 {
   SwfdecAsVariable *var = g_hash_table_lookup (object->properties, variable);
 
-  if (var || object->context->version >= 7)
+  if (var || swfdec_gc_object_get_context (object)->version >= 7)
     return var;
   var = g_hash_table_find (object->properties, swfdec_as_object_lookup_case_insensitive, (gpointer) variable);
   return var;
@@ -206,10 +271,9 @@ swfdec_as_object_hash_create (SwfdecAsObject *object, const char *variable, guin
 {
   SwfdecAsVariable *var;
 
-  if (!swfdec_as_context_use_mem (object->context, sizeof (SwfdecAsVariable)))
-    return NULL;
   if (!swfdec_as_variable_name_is_valid (variable))
     return NULL;
+  swfdec_as_context_use_mem (swfdec_gc_object_get_context (object), sizeof (SwfdecAsVariable));
   var = g_slice_new0 (SwfdecAsVariable);
   var->flags = flags;
   g_hash_table_insert (object->properties, (gpointer) variable, var);
@@ -246,12 +310,11 @@ swfdec_as_object_do_get (SwfdecAsObject *object, SwfdecAsObject *orig,
 
   /* variable flag checks */
   if (!swfdec_as_object_variable_enabled_in_version (var,
-	object->context->version))
+	swfdec_gc_object_get_context (object)->version))
     return FALSE;
 
   if (var->get) {
     swfdec_as_function_call (var->get, orig, 0, NULL, val);
-    swfdec_as_context_run (object->context);
     *flags = var->flags;
   } else {
     *val = var->value;
@@ -265,9 +328,8 @@ swfdec_as_watch_new (SwfdecAsFunction *function)
 {
   SwfdecAsWatch *watch;
 
-  if (!swfdec_as_context_use_mem (SWFDEC_AS_OBJECT (function)->context, 
-	sizeof (SwfdecAsWatch)))
-    return NULL;
+  swfdec_as_context_use_mem (swfdec_gc_object_get_context (function), 
+      sizeof (SwfdecAsWatch));
 
   watch = g_slice_new (SwfdecAsWatch);
   watch->refcount = 1;
@@ -276,36 +338,6 @@ swfdec_as_watch_new (SwfdecAsFunction *function)
   return watch;
 }
 
-static gboolean
-swfdec_as_watch_can_recurse (SwfdecAsWatch *watch)
-{
-  guint version;
-
-  version = SWFDEC_AS_OBJECT (watch->watch)->context->version;
-  if (version <= 6) {
-    return watch->refcount <= 1;
-  } else {
-    return watch->refcount <= 64 + 1;
-  }
-}
-
-static void
-swfdec_as_watch_ref (SwfdecAsWatch *watch)
-{
-  watch->refcount++;
-}
-
-static void
-swfdec_as_watch_unref (SwfdecAsWatch *watch)
-{
-  watch->refcount--;
-  if (watch->refcount == 0) {
-    swfdec_as_context_unuse_mem (SWFDEC_AS_OBJECT (watch->watch)->context, 
-	sizeof (SwfdecAsWatch));
-    g_slice_free (SwfdecAsWatch, watch);
-  }
-}
-
 /*
  * Like swfdec_as_object_get_prototype, but doesn't check 8_UP flag when
  * version is 7 and doesn't check if the property has been deleted if version
@@ -318,7 +350,7 @@ swfdec_as_object_get_prototype_internal (SwfdecAsObject *object)
 
   g_return_val_if_fail (SWFDEC_IS_AS_OBJECT (object), NULL);
 
-  version = object->context->version;
+  version = swfdec_gc_object_get_context (object)->version;
 
   if (object->prototype == NULL)
     return NULL;
@@ -351,7 +383,7 @@ swfdec_as_object_get_prototype (SwfdecAsObject *object)
 
   g_return_val_if_fail (SWFDEC_IS_AS_OBJECT (object), NULL);
 
-  version = object->context->version;
+  version = swfdec_gc_object_get_context (object)->version;
 
   prototype = swfdec_as_object_get_prototype_internal (object);
 
@@ -401,7 +433,7 @@ swfdec_as_object_hash_lookup_with_prototype (SwfdecAsObject *object,
     }
 
     if (i == SWFDEC_AS_OBJECT_PROTOTYPE_RECURSION_LIMIT) {
-      swfdec_as_context_abort (object->context, "Prototype recursion limit exceeded");
+      swfdec_as_context_abort (swfdec_gc_object_get_context (object), "Prototype recursion limit exceeded");
       return NULL;
     }
   }
@@ -424,12 +456,12 @@ swfdec_as_object_do_set (SwfdecAsObject *object, const char *variable,
     return;
 
   var = swfdec_as_object_hash_lookup_with_prototype (object, variable, &proto);
-  if (swfdec_as_context_is_aborted (object->context))
+  if (swfdec_as_context_is_aborted (swfdec_gc_object_get_context (object)))
     return;
 
   // if variable is disabled in this version
   if (var != NULL && !swfdec_as_object_variable_enabled_in_version (var,
-	object->context->version)) {
+	swfdec_gc_object_get_context (object)->version)) {
     if (proto == NULL) {
       // it's at the top level, remove getter and setter plus overwrite
       var->get = NULL;
@@ -448,7 +480,7 @@ swfdec_as_object_do_set (SwfdecAsObject *object, const char *variable,
     if (var->flags & SWFDEC_AS_VARIABLE_CONSTANT)
       return;
     // remove the flags that could make this variable hidden
-    if (object->context->version == 6) {
+    if (swfdec_gc_object_get_context (object)->version == 6) {
       // version 6, so let's forget SWFDEC_AS_VARIABLE_VERSION_7_UP flag, oops!
       // we will still set the value though, even if that flag is set
       var->flags &= ~(SWFDEC_AS_VARIABLE_VERSION_6_UP |
@@ -473,11 +505,10 @@ swfdec_as_object_do_set (SwfdecAsObject *object, const char *variable,
       args[3] = watch->watch_data;
       swfdec_as_watch_ref (watch);
       swfdec_as_function_call (watch->watch, object, 4, args, &ret);
-      swfdec_as_context_run (object->context);
       swfdec_as_watch_unref (watch);
       var = swfdec_as_object_hash_lookup_with_prototype (object, variable,
 	  NULL);
-      if (swfdec_as_context_is_aborted (object->context))
+      if (swfdec_as_context_is_aborted (swfdec_gc_object_get_context (object)))
 	return;
       if (var == NULL) {
 	SWFDEC_INFO ("watch removed variable %s", variable);
@@ -493,7 +524,6 @@ swfdec_as_object_do_set (SwfdecAsObject *object, const char *variable,
     if (var->set) {
       SwfdecAsValue tmp;
       swfdec_as_function_call (var->set, object, 1, val, &tmp);
-      swfdec_as_context_run (object->context);
     }
   } else if (watch == NULL) {
     var->value = *val;
@@ -524,15 +554,6 @@ swfdec_as_object_do_set_flags (SwfdecAsObject *object, const char *variable, gui
   }
 }
 
-static void
-swfdec_as_object_free_property (gpointer key, gpointer value, gpointer data)
-{
-  SwfdecAsObject *object = data;
-
-  swfdec_as_context_unuse_mem (object->context, sizeof (SwfdecAsVariable));
-  g_slice_free (SwfdecAsVariable, value);
-}
-
 static SwfdecAsDeleteReturn
 swfdec_as_object_do_delete (SwfdecAsObject *object, const char *variable)
 {
@@ -683,15 +704,37 @@ swfdec_as_object_do_debug (SwfdecAsObject *object)
   return g_strdup ("Object");
 }
 
+static GObject *
+swfdec_as_object_constructor (GType type, guint n_construct_properties,
+    GObjectConstructParam *construct_properties)
+{
+  GObject *gobject;
+  SwfdecAsContext *context;
+
+  gobject = G_OBJECT_CLASS (swfdec_as_object_parent_class)->constructor (type, 
+      n_construct_properties, construct_properties);
+
+  context = swfdec_gc_object_get_context (gobject);
+  if (context->debugger) {
+    SwfdecAsDebuggerClass *dklass = SWFDEC_AS_DEBUGGER_GET_CLASS (context->debugger);
+    if (dklass->add)
+      dklass->add (context->debugger, context, SWFDEC_AS_OBJECT (gobject));
+  }
+
+  return gobject;
+}
+
 static void
 swfdec_as_object_class_init (SwfdecAsObjectClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  SwfdecGcObjectClass *gc_class = SWFDEC_GC_OBJECT_CLASS (klass);
 
+  object_class->constructor = swfdec_as_object_constructor;
   object_class->dispose = swfdec_as_object_dispose;
 
-  klass->mark = swfdec_as_object_do_mark;
-  klass->add = swfdec_as_object_do_add;
+  gc_class->mark = swfdec_as_object_mark;
+
   klass->get = swfdec_as_object_do_get;
   klass->set = swfdec_as_object_do_set;
   klass->set_flags = swfdec_as_object_do_set_flags;
@@ -703,6 +746,7 @@ swfdec_as_object_class_init (SwfdecAsObjectClass *klass)
 static void
 swfdec_as_object_init (SwfdecAsObject *object)
 {
+  object->properties = g_hash_table_new (g_direct_hash, g_direct_equal);
 }
 
 /**
@@ -714,20 +758,14 @@ swfdec_as_object_init (SwfdecAsObject *object)
  * swfdec_as_object_set_constructor() on the returned object yourself.
  * You may want to use swfdec_as_object_new() instead.
  *
- * Returns: A new #SwfdecAsObject adde to @context or %NULL on OOM.
+ * Returns: A new #SwfdecAsObject added to @context
  **/
 SwfdecAsObject *
 swfdec_as_object_new_empty (SwfdecAsContext *context)
 {
-  SwfdecAsObject *object;
-
   g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), NULL);
   
-  if (!swfdec_as_context_use_mem (context, sizeof (SwfdecAsObject)))
-    return NULL;
-  object = g_object_new (SWFDEC_TYPE_AS_OBJECT, NULL);
-  swfdec_as_object_add (object, context, sizeof (SwfdecAsObject));
-  return object;
+  return g_object_new (SWFDEC_TYPE_AS_OBJECT, "context", context, NULL);
 }
 
 /**
@@ -737,7 +775,7 @@ swfdec_as_object_new_empty (SwfdecAsContext *context)
  * Allocates a new Object. This does the same as the Actionscript code 
  * "new Object()".
  *
- * Returns: the new object or NULL on out of memory.
+ * Returns: the new object
  **/
 SwfdecAsObject *
 swfdec_as_object_new (SwfdecAsContext *context)
@@ -750,8 +788,6 @@ swfdec_as_object_new (SwfdecAsContext *context)
   g_assert (context->Object_prototype);
   
   object = swfdec_as_object_new_empty (context);
-  if (object == NULL)
-    return NULL;
   SWFDEC_AS_VALUE_SET_OBJECT (&val, context->Object);
   swfdec_as_object_set_variable_and_flags (object, SWFDEC_AS_STR_constructor,
       &val, SWFDEC_AS_VARIABLE_HIDDEN | SWFDEC_AS_VARIABLE_PERMANENT);
@@ -761,73 +797,6 @@ swfdec_as_object_new (SwfdecAsContext *context)
   return object;
 }
 
-/**
- * swfdec_as_object_add:
- * @object: #SwfdecAsObject to make garbage-collected
- * @context: #SwfdecAsContext that should manage the object
- * @size: size the object currently uses
- *
- * Takes over the reference to @object for the garbage collector of @context. 
- * The object may not already be part of a different context. The given @size 
- * must have been allocated before with swfdec_as_context_use_mem ().
- * Note that after swfdec_as_object_add() the garbage collector might hold the
- * only reference to @object.
- **/
-void
-swfdec_as_object_add (SwfdecAsObject *object, SwfdecAsContext *context, gsize size)
-{
-  SwfdecAsObjectClass *klass;
-
-  g_return_if_fail (SWFDEC_IS_AS_OBJECT (object));
-  g_return_if_fail (SWFDEC_IS_AS_CONTEXT (context));
-  g_return_if_fail (object->properties == NULL);
-
-  object->context = context;
-  object->size = size;
-  g_hash_table_insert (context->objects, object, object);
-  object->properties = g_hash_table_new (g_direct_hash, g_direct_equal);
-  klass = SWFDEC_AS_OBJECT_GET_CLASS (object);
-  g_return_if_fail (klass->add);
-  klass->add (object);
-  if (context->debugger) {
-    SwfdecAsDebuggerClass *dklass = SWFDEC_AS_DEBUGGER_GET_CLASS (context->debugger);
-    if (dklass->add)
-      dklass->add (context->debugger, context, object);
-  }
-}
-
-/* This is a huge hack design-wise, but we can't use watch->watch, 
- * it might be gone already */
-static gboolean
-swfdec_as_object_steal_watches (gpointer key, gpointer value, gpointer object)
-{
-  SwfdecAsWatch *watch = value;
-
-  g_assert (watch->refcount == 1);
-  watch->watch = (SwfdecAsFunction *) object;
-  swfdec_as_watch_unref (watch);
-  return TRUE;
-}
-
-void
-swfdec_as_object_collect (SwfdecAsObject *object)
-{
-  g_return_if_fail (SWFDEC_IS_AS_OBJECT (object));
-  g_return_if_fail (object->properties != NULL);
-
-  g_hash_table_foreach (object->properties, swfdec_as_object_free_property, object);
-  g_hash_table_destroy (object->properties);
-  object->properties = NULL;
-  if (object->watches) {
-    g_hash_table_foreach_steal (object->watches, swfdec_as_object_steal_watches, object);
-    g_hash_table_destroy (object->watches);
-    object->watches = NULL;
-  }
-  if (object->size)
-    swfdec_as_context_unuse_mem (object->context, object->size);
-  g_object_unref (object);
-}
-
 /**
  * swfdec_as_object_set_variable:
  * @object: a #SwfdecAsObject
@@ -861,11 +830,11 @@ swfdec_as_object_set_variable_and_flags (SwfdecAsObject *object,
   g_return_if_fail (variable != NULL);
   g_return_if_fail (SWFDEC_IS_AS_VALUE (value));
 
-  if (object->context->debugger) {
-    SwfdecAsDebugger *debugger = object->context->debugger;
+  if (swfdec_gc_object_get_context (object)->debugger) {
+    SwfdecAsDebugger *debugger = swfdec_gc_object_get_context (object)->debugger;
     SwfdecAsDebuggerClass *dklass = SWFDEC_AS_DEBUGGER_GET_CLASS (debugger);
     if (dklass->set_variable)
-      dklass->set_variable (debugger, object->context, object, variable, value);
+      dklass->set_variable (debugger, swfdec_gc_object_get_context (object), object, variable, value);
   }
   klass = SWFDEC_AS_OBJECT_GET_CLASS (object);
   klass->set (object, variable, value, default_flags);
@@ -935,7 +904,7 @@ swfdec_as_object_get_variable_and_flags (SwfdecAsObject *object,
   guint i;
   SwfdecAsValue tmp_val;
   guint tmp_flags;
-  SwfdecAsObject *tmp_pobject, *cur;
+  SwfdecAsObject *tmp_pobject, *cur, *resolve;
 
   g_return_val_if_fail (SWFDEC_IS_AS_OBJECT (object), FALSE);
   g_return_val_if_fail (variable != NULL, FALSE);
@@ -948,20 +917,53 @@ swfdec_as_object_get_variable_and_flags (SwfdecAsObject *object,
     pobject = &tmp_pobject;
 
   cur = object;
+  resolve = NULL;
   for (i = 0; i <= SWFDEC_AS_OBJECT_PROTOTYPE_RECURSION_LIMIT && cur != NULL; i++) {
     klass = SWFDEC_AS_OBJECT_GET_CLASS (cur);
     if (klass->get (cur, object, variable, value, flags)) {
       *pobject = cur;
       return TRUE;
     }
+    if (resolve == NULL) {
+      SwfdecAsVariable *var =
+	swfdec_as_object_hash_lookup (cur, SWFDEC_AS_STR___resolve);
+
+      if (var != NULL && (swfdec_gc_object_get_context (object)->version <= 6 ||
+	    SWFDEC_AS_VALUE_IS_OBJECT (&var->value)))
+	resolve = cur;
+    }
     cur = swfdec_as_object_get_prototype_internal (cur);
   }
   if (i > SWFDEC_AS_OBJECT_PROTOTYPE_RECURSION_LIMIT) {
-    swfdec_as_context_abort (object->context, "Prototype recursion limit exceeded");
+    swfdec_as_context_abort (swfdec_gc_object_get_context (object), "Prototype recursion limit exceeded");
+    SWFDEC_AS_VALUE_SET_UNDEFINED (value);
     *flags = 0;
     *pobject = NULL;
     return FALSE;
   }
+  if (variable != SWFDEC_AS_STR___resolve && resolve != NULL) {
+    SwfdecAsValue argv;
+    SwfdecAsVariable *var;
+    SwfdecAsFunction *fun;
+    SwfdecAsContext *context;
+
+    *flags = 0;
+    *pobject = resolve;
+    SWFDEC_AS_VALUE_SET_UNDEFINED (value);
+    context = swfdec_gc_object_get_context (resolve);
+
+    var = swfdec_as_object_hash_lookup (resolve, SWFDEC_AS_STR___resolve);
+    g_assert (var != NULL);
+    if (!SWFDEC_AS_VALUE_IS_OBJECT (&var->value))
+      return FALSE;
+    fun = (SwfdecAsFunction *) SWFDEC_AS_VALUE_GET_OBJECT (&var->value);
+    if (!SWFDEC_IS_AS_FUNCTION (fun))
+      return FALSE;
+    SWFDEC_AS_VALUE_SET_STRING (&argv, variable);
+    swfdec_as_function_call (fun, resolve, 1, &argv, value);
+
+    return TRUE;
+  }
   //SWFDEC_WARNING ("no such variable %s", variable);
   SWFDEC_AS_VALUE_SET_UNDEFINED (value);
   *flags = 0;
@@ -1118,25 +1120,20 @@ swfdec_as_object_do_nothing (SwfdecAsContext *cx, SwfdecAsObject *object,
  * @object: a #SwfdecAsObject
  * @name: name of the function. The string does not have to be 
  *        garbage-collected.
- * @type: the required type of the this Object to make this function execute.
- *        May be 0 to accept any type.
  * @native: a native function or %NULL to just not do anything
- * @min_args: minimum number of arguments to pass to @native
  *
  * Adds @native as a variable named @name to @object. The newly added variable
  * will not be enumerated.
  *
- * Returns: the newly created #SwfdecAsFunction or %NULL on error.
+ * Returns: the newly created #SwfdecAsFunction
  **/
 SwfdecAsFunction *
-swfdec_as_object_add_function (SwfdecAsObject *object, const char *name, GType type,
-    SwfdecAsNative native, guint min_args)
+swfdec_as_object_add_function (SwfdecAsObject *object, const char *name, SwfdecAsNative native)
 {
   g_return_val_if_fail (SWFDEC_IS_AS_OBJECT (object), NULL);
   g_return_val_if_fail (name != NULL, NULL);
-  g_return_val_if_fail (type == 0 || g_type_is_a (type, SWFDEC_TYPE_AS_OBJECT), NULL);
 
-  return swfdec_as_object_add_constructor (object, name, type, 0, native, min_args, NULL);
+  return swfdec_as_object_add_constructor (object, name, 0, native, NULL);
 }
 
 /**
@@ -1144,43 +1141,35 @@ swfdec_as_object_add_function (SwfdecAsObject *object, const char *name, GType t
  * @object: a #SwfdecAsObject
  * @name: name of the function. The string does not have to be 
  *        garbage-collected.
- * @type: the required type of the this Object to make this function execute.
- *        May be 0 to accept any type.
  * @construct_type: type used when using this function as a constructor. May 
  *                  be 0 to use the default type.
  * @native: a native function or %NULL to just not do anything
- * @min_args: minimum number of arguments to pass to @native
  * @prototype: An optional object to be set as the "prototype" property of the
  *             new function. The prototype will be hidden and constant.
  *
  * Adds @native as a constructor named @name to @object. The newly added variable
  * will not be enumerated.
  *
- * Returns: the newly created #SwfdecAsFunction or %NULL on error.
+ * Returns: the newly created #SwfdecAsFunction
  **/
 SwfdecAsFunction *
-swfdec_as_object_add_constructor (SwfdecAsObject *object, const char *name, GType type,
-    GType construct_type, SwfdecAsNative native, guint min_args, SwfdecAsObject *prototype)
+swfdec_as_object_add_constructor (SwfdecAsObject *object, const char *name,
+    GType construct_type, SwfdecAsNative native, SwfdecAsObject *prototype)
 {
   SwfdecAsFunction *function;
   SwfdecAsValue val;
 
   g_return_val_if_fail (SWFDEC_IS_AS_OBJECT (object), NULL);
   g_return_val_if_fail (name != NULL, NULL);
-  g_return_val_if_fail (type == 0 || g_type_is_a (type, SWFDEC_TYPE_AS_OBJECT), NULL);
   g_return_val_if_fail (construct_type == 0 || g_type_is_a (construct_type, SWFDEC_TYPE_AS_OBJECT), NULL);
   g_return_val_if_fail (prototype == NULL || SWFDEC_IS_AS_OBJECT (prototype), NULL);
 
   if (!native)
     native = swfdec_as_object_do_nothing;
-  function = swfdec_as_native_function_new (object->context, name, native, min_args, prototype);
-  if (function == NULL)
-    return NULL;
-  if (type != 0)
-    swfdec_as_native_function_set_object_type (SWFDEC_AS_NATIVE_FUNCTION (function), type);
+  function = swfdec_as_native_function_new (swfdec_gc_object_get_context (object), name, native, prototype);
   if (construct_type != 0)
     swfdec_as_native_function_set_construct_type (SWFDEC_AS_NATIVE_FUNCTION (function), construct_type);
-  name = swfdec_as_context_get_string (object->context, name);
+  name = swfdec_as_context_get_string (swfdec_gc_object_get_context (object), name);
   SWFDEC_AS_VALUE_SET_OBJECT (&val, SWFDEC_AS_OBJECT (function));
   /* FIXME: I'd like to make sure no such property exists yet */
   swfdec_as_object_set_variable_and_flags (object, name, &val,
@@ -1198,18 +1187,18 @@ swfdec_as_object_add_constructor (SwfdecAsObject *object, const char *name, GTyp
 void
 swfdec_as_object_run (SwfdecAsObject *object, SwfdecScript *script)
 {
+  SwfdecAsFrame frame = { NULL, };
   SwfdecAsContext *context;
-  SwfdecAsFrame *frame;
 
   g_return_if_fail (SWFDEC_IS_AS_OBJECT (object));
   g_return_if_fail (script != NULL);
 
-  context = object->context;
-  frame = swfdec_as_frame_new (context, script);
-  if (frame == NULL)
-    return;
-  swfdec_as_frame_set_this (frame, object);
-  swfdec_as_frame_preload (frame);
+  context = swfdec_gc_object_get_context (object);
+  swfdec_as_frame_init (&frame, context, script);
+  swfdec_as_frame_set_this (&frame, object);
+  swfdec_as_frame_preload (&frame);
+  /* we take no prisoners */
+  frame.activation = NULL;
   swfdec_as_context_run (context);
   swfdec_as_stack_pop (context);
 }
@@ -1228,31 +1217,32 @@ swfdec_as_object_run (SwfdecAsObject *object, SwfdecScript *script)
  * <informalexample><programlisting>
  * @return_value = @object.@name (@argv[0], ..., @argv[argc-1]);
  * </programlisting></informalexample>
+ *
+ * Returns: %TRUE if @object had a function with the given name, %FALSE otherwise
  **/
-void
+gboolean
 swfdec_as_object_call (SwfdecAsObject *object, const char *name, guint argc, 
     SwfdecAsValue *argv, SwfdecAsValue *return_value)
 {
   SwfdecAsValue tmp;
   SwfdecAsFunction *fun;
 
-  g_return_if_fail (SWFDEC_IS_AS_OBJECT (object));
-  g_return_if_fail (name != NULL);
-  g_return_if_fail (argc == 0 || argv != NULL);
-  g_return_if_fail (object->context->global != NULL); /* for SwfdecPlayer */
+  g_return_val_if_fail (SWFDEC_IS_AS_OBJECT (object), TRUE);
+  g_return_val_if_fail (name != NULL, TRUE);
+  g_return_val_if_fail (argc == 0 || argv != NULL, TRUE);
+  g_return_val_if_fail (swfdec_gc_object_get_context (object)->global != NULL, TRUE); /* for SwfdecPlayer */
 
   if (return_value)
     SWFDEC_AS_VALUE_SET_UNDEFINED (return_value);
   swfdec_as_object_get_variable (object, name, &tmp);
   if (!SWFDEC_AS_VALUE_IS_OBJECT (&tmp))
-    return;
+    return FALSE;
   fun = (SwfdecAsFunction *) SWFDEC_AS_VALUE_GET_OBJECT (&tmp);
   if (!SWFDEC_IS_AS_FUNCTION (fun))
-    return;
+    return FALSE;
   swfdec_as_function_call (fun, object, argc, argv, return_value ? return_value : &tmp);
-  if (swfdec_as_context_is_aborted (object->context))
-    return;
-  swfdec_as_context_run (object->context);
+
+  return TRUE;
 }
 
 /**
@@ -1263,9 +1253,7 @@ swfdec_as_object_call (SwfdecAsObject *object, const char *name, guint argc,
  * @return_value: pointer for return value or %NULL to push the return value to 
  *                the stack
  *
- * Creates a new object for the given constructor and pushes the constructor on
- * top of the stack. To actually run the constructor, you need to call 
- * swfdec_as_context_run().
+ * Creates a new object for the given constructor and runs the constructor.
  **/
 void
 swfdec_as_object_create (SwfdecAsFunction *fun, guint n_args, 
@@ -1275,14 +1263,18 @@ swfdec_as_object_create (SwfdecAsFunction *fun, guint n_args,
   SwfdecAsObject *new;
   SwfdecAsContext *context;
   SwfdecAsFunction *cur;
-  SwfdecAsFrame *frame;
-  guint size = 0;
+  guint i, size = 0;
   GType type = 0;
 
   g_return_if_fail (SWFDEC_IS_AS_FUNCTION (fun));
 
-  context = SWFDEC_AS_OBJECT (fun)->context;
+  // FIXME: The way we decide object's type is wrong
+  // It seems to be actually possible to change the type inside a constructor
+  // (many times) by changing this.__proto__.__constructor__ and calling super
+
+  context = swfdec_gc_object_get_context (fun);
   cur = fun;
+  i = 0;
   do {
     if (SWFDEC_IS_AS_NATIVE_FUNCTION (cur)) {
       SwfdecAsNativeFunction *native = SWFDEC_AS_NATIVE_FUNCTION (cur);
@@ -1292,6 +1284,7 @@ swfdec_as_object_create (SwfdecAsFunction *fun, guint n_args,
 	break;
       }
     }
+    i++;
     swfdec_as_object_get_variable (SWFDEC_AS_OBJECT (cur), SWFDEC_AS_STR_prototype, &val);
     if (SWFDEC_AS_VALUE_IS_OBJECT (&val)) {
       SwfdecAsObject *proto = SWFDEC_AS_VALUE_GET_OBJECT (&val);
@@ -1304,16 +1297,13 @@ swfdec_as_object_create (SwfdecAsFunction *fun, guint n_args,
       }
     }
     cur = NULL;
-  } while (type == 0 && cur != NULL);
+  } while (type == 0 && cur != NULL && i < SWFDEC_AS_OBJECT_PROTOTYPE_RECURSION_LIMIT);
   if (type == 0) {
     type = SWFDEC_TYPE_AS_OBJECT;
     size = sizeof (SwfdecAsObject);
   }
-  if (!swfdec_as_context_use_mem (context, size))
-    return;
 
-  new = g_object_new (type, NULL);
-  swfdec_as_object_add (new, context, size);
+  new = g_object_new (type, "context", context, NULL);
   /* set initial variables */
   if (swfdec_as_object_get_variable (SWFDEC_AS_OBJECT (fun), SWFDEC_AS_STR_prototype, &val)) {
       swfdec_as_object_set_variable_and_flags (new, SWFDEC_AS_STR___proto__,
@@ -1327,10 +1317,34 @@ swfdec_as_object_create (SwfdecAsFunction *fun, guint n_args,
   swfdec_as_object_set_variable_and_flags (new, SWFDEC_AS_STR___constructor__, 
       &val, SWFDEC_AS_VARIABLE_HIDDEN | SWFDEC_AS_VARIABLE_VERSION_6_UP);
 
-  frame = swfdec_as_function_call_no_preload (fun, new, n_args, args, return_value);
-  frame->construct = TRUE;
-  swfdec_as_super_new (frame, new, new->prototype);
-  swfdec_as_frame_preload (frame);
+  swfdec_as_function_call_full (fun, new, TRUE, new->prototype, n_args, args, return_value);
+}
+
+void
+swfdec_as_object_set_constructor_by_name (SwfdecAsObject *object, const char *name, ...)
+{
+  SwfdecAsContext *context;
+  SwfdecAsObject *cur;
+  SwfdecAsValue val;
+  va_list args;
+
+  g_return_if_fail (SWFDEC_IS_AS_OBJECT (object));
+  g_return_if_fail (name != NULL);
+
+  context = swfdec_gc_object_get_context (object);
+  va_start (args, name);
+  cur = context->global;
+  do {
+    if (!swfdec_as_object_get_variable (cur, name, &val) || 
+	!SWFDEC_AS_VALUE_IS_OBJECT (&val)) {
+      SWFDEC_WARNING ("could not find constructor %s", name);
+      return;
+    }
+    cur = SWFDEC_AS_VALUE_GET_OBJECT (&val);
+    name = va_arg (args, const char *);
+  } while (name != NULL);
+  va_end (args);
+  swfdec_as_object_set_constructor (object, cur);
 }
 
 /**
@@ -1413,13 +1427,13 @@ swfdec_as_object_add_native_variable (SwfdecAsObject *object,
   g_return_if_fail (get != NULL);
 
   get_func =
-    swfdec_as_native_function_new (object->context, variable, get, 0, NULL);
+    swfdec_as_native_function_new (swfdec_gc_object_get_context (object), variable, get, NULL);
   if (get_func == NULL)
     return;
 
   if (set != NULL) {
     set_func =
-      swfdec_as_native_function_new (object->context, variable, set, 0, NULL);
+      swfdec_as_native_function_new (swfdec_gc_object_get_context (object), variable, set, NULL);
   } else {
     set_func = NULL;
   }
@@ -1475,7 +1489,7 @@ swfdec_as_object_hasOwnProperty (SwfdecAsContext *cx, SwfdecAsObject *object,
   if (argc < 1)
     return;
 
-  name = swfdec_as_value_to_string (object->context, &argv[0]);
+  name = swfdec_as_value_to_string (swfdec_gc_object_get_context (object), &argv[0]);
 
   if (!(var = swfdec_as_object_hash_lookup (object, name)))
     return;
@@ -1505,7 +1519,7 @@ swfdec_as_object_isPropertyEnumerable (SwfdecAsContext *cx,
   if (argc < 1)
     return;
 
-  name = swfdec_as_value_to_string (object->context, &argv[0]);
+  name = swfdec_as_value_to_string (swfdec_gc_object_get_context (object), &argv[0]);
 
   if (!(var = swfdec_as_object_hash_lookup (object, name)))
     return;
@@ -1657,7 +1671,7 @@ swfdec_as_object_old_constructor (SwfdecAsContext *cx, SwfdecAsObject *object,
 void
 swfdec_as_object_decode (SwfdecAsObject *object, const char *str)
 {
-  SwfdecAsContext *cx = object->context;
+  SwfdecAsContext *cx = swfdec_gc_object_get_context (object);
   SwfdecAsValue val;
   char **varlist, *p, *unescaped;
   guint i;
@@ -1695,6 +1709,29 @@ swfdec_as_object_decode (SwfdecAsObject *object, const char *str)
   g_strfreev (varlist);
 }
 
+static void
+swfdec_as_object_construct (SwfdecAsContext *cx, SwfdecAsObject *object,
+    guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
+{
+  if (argc > 0) {
+    SwfdecAsObject *result = swfdec_as_value_to_object (cx, &argv[0]);
+    if (result != NULL) {
+      if (!cx->frame->construct) {
+	SWFDEC_AS_VALUE_SET_OBJECT (ret, result);
+      } else {
+	SWFDEC_FIXME ("new Object (x) should return x");
+	SWFDEC_AS_VALUE_SET_OBJECT (ret, object);
+      }
+      return;
+    }
+  }
+
+  if (!cx->frame->construct)
+    object = swfdec_as_object_new_empty (cx);
+
+  SWFDEC_AS_VALUE_SET_OBJECT (ret, object);
+}
+
 void
 swfdec_as_object_init_context (SwfdecAsContext *context)
 {
@@ -1702,12 +1739,8 @@ swfdec_as_object_init_context (SwfdecAsContext *context)
   SwfdecAsObject *object, *proto;
 
   proto = swfdec_as_object_new_empty (context);
-  if (!proto)
-    return;
   object = SWFDEC_AS_OBJECT (swfdec_as_object_add_function (context->global, 
-      SWFDEC_AS_STR_Object, 0, NULL, 0));
-  if (!object)
-    return;
+      SWFDEC_AS_STR_Object, swfdec_as_object_construct));
   context->Object = object;
   context->Object_prototype = proto;
   SWFDEC_AS_VALUE_SET_OBJECT (&val, proto);
