@@ -39,6 +39,7 @@
 #include "swfdec_rectangle.h"
 #include "swfdec_renderer_internal.h"
 #include "swfdec_resource.h"
+#include "swfdec_utils.h"
 
 enum {
   INVALIDATE,
@@ -106,7 +107,6 @@ SwfdecBitmapData *
 swfdec_bitmap_data_new (SwfdecAsContext *context, gboolean transparent, guint width, guint height)
 {
   SwfdecBitmapData *bitmap;
-  SwfdecAsValue val;
 
   g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), NULL);
   g_return_val_if_fail (width > 0, NULL);
@@ -119,19 +119,8 @@ swfdec_bitmap_data_new (SwfdecAsContext *context, gboolean transparent, guint wi
   bitmap->surface = cairo_image_surface_create (
       transparent ? CAIRO_FORMAT_ARGB32 : CAIRO_FORMAT_RGB24, width, height);
 
-  swfdec_as_object_get_variable (context->global, SWFDEC_AS_STR_flash, &val);
-  if (SWFDEC_AS_VALUE_IS_OBJECT (&val)) {
-    swfdec_as_object_get_variable (SWFDEC_AS_VALUE_GET_OBJECT (&val), 
-	SWFDEC_AS_STR_display, &val);
-    if (SWFDEC_AS_VALUE_IS_OBJECT (&val)) {
-      swfdec_as_object_get_variable (SWFDEC_AS_VALUE_GET_OBJECT (&val), 
-	  SWFDEC_AS_STR_BitmapData, &val);
-      if (SWFDEC_AS_VALUE_IS_OBJECT (&val)) {
-	swfdec_as_object_set_constructor (SWFDEC_AS_OBJECT (bitmap),
-	    SWFDEC_AS_VALUE_GET_OBJECT (&val));
-      }
-    }
-  }
+  swfdec_as_object_set_constructor_by_name (SWFDEC_AS_OBJECT (bitmap),
+      SWFDEC_AS_STR_flash, SWFDEC_AS_STR_display, SWFDEC_AS_STR_BitmapData, NULL);
 
   return bitmap;
 }
@@ -257,7 +246,6 @@ swfdec_bitmap_data_get_rectangle (SwfdecAsContext *cx, SwfdecAsObject *object,
   SWFDEC_AS_VALUE_SET_INT (&args[2], cairo_image_surface_get_width (bitmap->surface));
   SWFDEC_AS_VALUE_SET_INT (&args[3], cairo_image_surface_get_width (bitmap->surface));
   swfdec_as_object_create (SWFDEC_AS_FUNCTION (o), 4, args, ret);
-  swfdec_as_context_run (cx);
 }
 
 SWFDEC_AS_NATIVE (1100, 105, swfdec_bitmap_data_set_rectangle)
@@ -406,10 +394,10 @@ void
 swfdec_bitmap_data_copyPixels (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
-  SwfdecBitmapData *bitmap, *source, *alpha;
-  SwfdecAsObject *recto, *apt, *pt;
+  SwfdecBitmapData *bitmap, *source, *alpha = NULL;
+  SwfdecAsObject *recto, *pt, *apt = NULL;
   SwfdecRectangle rect;
-  gboolean copy_alpha;
+  gboolean copy_alpha = FALSE;
   cairo_t *cr;
   int x, y;
 
@@ -460,7 +448,6 @@ swfdec_bitmap_data_copyPixels (SwfdecAsContext *cx, SwfdecAsObject *object,
     }
     cairo_mask_surface (cr2, alpha->surface, rect.x - x, rect.y - y);
     cairo_destroy (cr2);
-    cairo_surface_write_to_png (mask, "foo.png");
     cairo_set_source_surface (cr, mask, 0, 0);
     cairo_surface_destroy (mask);
   }
@@ -494,51 +481,12 @@ swfdec_bitmap_data_threshold (SwfdecAsContext *cx, SwfdecAsObject *object,
   SWFDEC_STUB ("BitmapData.threshold");
 }
 
-static gboolean
-swfdec_matrix_from_as_object (cairo_matrix_t *matrix, SwfdecAsObject *object)
-{
-  SwfdecAsValue *val;
-  SwfdecAsContext *cx = swfdec_gc_object_get_context (object);
-
-  val = swfdec_as_object_peek_variable (object, SWFDEC_AS_STR_a);
-  if (val == NULL ||
-      !isfinite (matrix->xx = swfdec_as_value_to_number (cx, val)))
-    return FALSE;
-  val = swfdec_as_object_peek_variable (object, SWFDEC_AS_STR_b);
-  if (val == NULL ||
-      !isfinite (matrix->yx = swfdec_as_value_to_number (cx, val)))
-    return FALSE;
-  val = swfdec_as_object_peek_variable (object, SWFDEC_AS_STR_c);
-  if (val == NULL ||
-      !isfinite (matrix->xy = swfdec_as_value_to_number (cx, val)))
-    return FALSE;
-  val = swfdec_as_object_peek_variable (object, SWFDEC_AS_STR_d);
-  if (val == NULL ||
-      !isfinite (matrix->yy = swfdec_as_value_to_number (cx, val)))
-    return FALSE;
-
-  val = swfdec_as_object_peek_variable (object, SWFDEC_AS_STR_tx);
-  if (val == NULL)
-      return FALSE;
-  matrix->x0 = swfdec_as_value_to_number (cx, val);
-  if (!isfinite (matrix->x0))
-    matrix->x0 = 0;
-  val = swfdec_as_object_peek_variable (object, SWFDEC_AS_STR_ty);
-  if (val == NULL)
-      return FALSE;
-  matrix->y0 = swfdec_as_value_to_number (cx, val);
-  if (!isfinite (matrix->y0))
-    matrix->y0 = 0;
-
-  return TRUE;
-}
-
 SWFDEC_AS_NATIVE (1100, 8, swfdec_bitmap_data_draw)
 void
 swfdec_bitmap_data_draw (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
-  SwfdecAsObject *o, *matrix, *trans;
+  SwfdecAsObject *o, *matrix = NULL, *trans = NULL;
   cairo_t *cr;
   SwfdecColorTransform ctrans;
   SwfdecBitmapData *bitmap;
@@ -547,6 +495,9 @@ swfdec_bitmap_data_draw (SwfdecAsContext *cx, SwfdecAsObject *object,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_BITMAP_DATA, &bitmap, "o|OO", &o, &matrix, &trans);
 
+  if (bitmap->surface == NULL)
+    return;
+
   if (argc >= 2) {
     if (matrix == NULL || !swfdec_matrix_from_as_object (&mat, matrix))
       return;
@@ -585,8 +536,11 @@ swfdec_bitmap_data_draw (SwfdecAsContext *cx, SwfdecAsObject *object,
       cairo_paint (cr);
     }
   } else if (SWFDEC_IS_MOVIE (o)) {
+    SwfdecMovie *movie = SWFDEC_MOVIE (o);
+    swfdec_movie_update (movie);
     cairo_scale (cr, 1.0 / SWFDEC_TWIPS_SCALE_FACTOR, 1.0 / SWFDEC_TWIPS_SCALE_FACTOR);
-    swfdec_movie_render (SWFDEC_MOVIE (o), cr, &ctrans);
+    cairo_transform (cr, &movie->inverse_matrix);
+    swfdec_movie_render (movie, cr, &ctrans);
   } else {
     SWFDEC_FIXME ("BitmapData.draw() with a %s?", G_OBJECT_TYPE_NAME (o));
   }
@@ -791,8 +745,8 @@ swfdec_bitmap_data_construct (SwfdecAsContext *cx, SwfdecAsObject *object,
 {
   SwfdecBitmapData *bitmap;
   int w, h;
-  gboolean transparent;
-  guint color;
+  gboolean transparent = TRUE;
+  guint color = 0;
 
   if (!swfdec_as_context_is_constructing (cx))
     return;
@@ -804,8 +758,6 @@ swfdec_bitmap_data_construct (SwfdecAsContext *cx, SwfdecAsObject *object,
     SWFDEC_FIXME ("the constructor should return undefined here");
     return;
   }
-  if (argc < 3)
-    transparent = TRUE;
 
   if (!swfdec_as_context_try_use_mem (cx, w * h * 4))
     return;
